/**
 *	ScanChannelsBDA.cpp
 *	Copyright (C) 2007 Chris Day 
 *	Copyright (C) 2004 BionicDonkey
 *  Copyright (C) 2004 Nate
 *  Copyright (C) 2004 JoeyBloggs
 *
 *	This file is part of DigitalWatch, a free DTV watching and recording
 *	program for the VisionPlus DVB-T.
 *
 *	DigitalWatch is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	DigitalWatch is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with DigitalWatch; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stdafx.h"
#include "ScanChannelsBDA.h"
#include "LogMessage.h"
#include "GlobalFunctions.h"

#include <ks.h> // Must be included before ksmedia.h
#include <ksmedia.h> // Must be included before bdamedia.h
#include <bdamedia.h>
#include <stdio.h>
#include <stdlib.h>

extern LogMessageWriter lmw;

#define STATCOUNT	20		// No. attempts to read stats. default 20
#define STATTIME	500		// ms between stat reads. default 500
#define SCANCOUNT	20		// No. of samples for signal quality. default 20
#define SCANTIME	1500	// ms between signal quality samples. default 1500


/*=======================================================
remove this for now
//=========================================================
//code for summary
#define N_SUMMARY 5

struct scan_summary 
{
	string summary_trans;
	string summary_mux;
	int summary_freq;
	int summary_str;
	int summary_qual;
} summary [N_SUMMARY];

int summ;	

void printsummary(scan_summary summaries)
{
	cout << "Frequency: " << summaries.summary_freq << "kHz,  Strength: " << summaries.summary_str << ",   Quality: " << summaries.summary_qual << endl;
	//(log << "Frequency: " << summaries.summary_freq << "kHz,  Strength: " << summaries.summary_str << ",   Quality: " << summaries.summary_qual).Write();
	//	cout << summaries.summary_trans;
//	cout << summaries.summary_mux;
}
===================================================
*/

BDAChannelScan::BDAChannelScan()
{
	CoInitializeEx(NULL, COINIT_MULTITHREADED);

	m_bServiceChanged = FALSE;
	m_bProgramChanged = FALSE;
	m_bScheduleEntryChanged = FALSE;

	m_hGuideDataChangedEvent = CreateEvent(NULL, TRUE, FALSE, "GuideDataChangedEvent");
	m_hServiceChangedMutex = CreateMutex(NULL, FALSE, NULL);

	m_pBDACard = NULL;

	m_rotEntry = 0;

	m_Count = 0;
	m_bScanning = FALSE;
	m_bVerbose = FALSE;
	m_nLockDetectionMode = 0;

	cardList.SetLogCallback(&lmw);
	SetLogCallback(&m_console);
	SetLogCallback(&lmw);
	m_consoleHandle = m_mpeg2parser.Output()->AddCallback(&m_console);
	m_logfileHandle = m_mpeg2parser.Output()->AddCallback(&lmw);
	verbose.AddCallback(&lmw);

  m_bSageTVOutputChannelList = true;
  m_bSageTVUseExactFrequency = false;
  m_bSageTVIncludeRadio = true;
  m_bSageTVSortByLCN = true;
	m_bSageTVMoveDuplicates = false;
}

BDAChannelScan::~BDAChannelScan()
{
	m_mpeg2parser.Output()->RemoveCallback(m_consoleHandle);
	m_mpeg2parser.Output()->RemoveCallback(m_logfileHandle);
	CoUninitialize();
}

HRESULT BDAChannelScan::selectCard()
{
	cardList.LoadCards();

	int cardsFound = cardList.cards.size();
	if (cardsFound == 0)
	{
		return (log << "\nCould not find any BDA devices. Check that drivers are installed and that the TV tuner is connected to your PC.\n").Write(E_FAIL);
	}

	BDACard *bdaCard = NULL;

	if (cardsFound == 1)
	{
		m_pBDACard = *(cardList.cards.begin());
		return (log << "Found BDA device: " << m_pBDACard->tunerDevice.strFriendlyName << "\n").Write(S_OK);
	}

	(log << cardsFound << " BDA devices found.\n").Show();

	std::vector<BDACard *>::iterator it;
	int id, cardID;

	while (TRUE)
	{
		cout << "Please select which device to use." << endl;

		id = 1;
		it = cardList.cards.begin();
		for ( ; it != cardList.cards.end() ; it++ )
		{
			bdaCard = *it;
			printf("%i. %S\n", id, bdaCard->tunerDevice.strFriendlyName);
			id++;
		}
		cout << id << ". Exit" << endl;
		cout << "> ";
		cin >> cardID;

		//Did Exit get chosen?
		if (cardID-1 == cardsFound)
		{
			return (log << "Exit chosen\n").Write(E_FAIL);
		}

		//check for invalid selection
		if ((cardID > cardsFound) || (cardID <= 0))
		{
			cout << "Invalid device selection. Please try again." << endl;
			continue;
		}

		id = 1;
		it = cardList.cards.begin();
		for ( ; it != cardList.cards.end() ; it++ )
		{
			if (id == cardID)
			{
				m_pBDACard = *it;
				return (log << "using " << m_pBDACard->tunerDevice.strFriendlyName << "\n").Write(S_OK);
			}
			id++;
		}
		(log << "Weird error. Could not find selected device in list\n").Write();
	}

	return E_FAIL;
}

void BDAChannelScan::AddNetwork(long freq, long band)
{
	if (m_Count < 256)
	{
		(log << "Adding network " << freq << ", " << band << "\n").Write();
		m_freq[m_Count] = freq;
		m_band[m_Count] = band;
		m_Count++;
	}
	else
	{
		(log << "Cannot add more than 256 networks\n").Write();
	}
}

HRESULT BDAChannelScan::SetupGraph()
{
	HRESULT hr = S_OK;

	if (FAILED(hr = CreateGraph()))
	{
		return (log << "Failed to create graph.\n").Show(hr);
	}
	
	if (FAILED(hr = BuildGraph()))
	{
		return (log << "Failed to add filters to graph.\n").Show(hr);
	}

	if (FAILED(hr = ConnectGraph()))
	{
		return (log << "Failed to connect filters in graph.\n").Show(hr);
	}

	if (FAILED(hr = createConnectionPoint()))
	{
		return (log << "Failed to add connection point to TIF.\n").Show(hr);
	}

	if (StartGraph() == FALSE)
	{
		return (log << "Failed to start the graph\n").Show(hr);
	}

	return hr;
}

void BDAChannelScan::DestroyGraph()
{
	StopGraph();

	if (m_piConnectionPoint)
		m_piConnectionPoint->Unadvise(m_dwAdviseCookie); 

	m_piConnectionPoint.Release();
	//m_piGuideData.Release();

	graphTools.DisconnectAllPins(m_piGraphBuilder);

	if (m_pBDACard)
		m_pBDACard->RemoveFilters();

	m_pTuningSpace.Release();
	m_piTuner.Release();
	graphTools.RemoveAllFilters(m_piGraphBuilder);

	m_pBDASecTab.Release();
	m_pBDATIF.Release();
	m_pBDAMpeg2Demux.Release();

	try
	{
		m_pBDANetworkProvider.Release();	//causes an exception. don't know why
		//m_pBDANetworkProvider.Detach();		//so i'll just do this instead
	}
	catch (...)
	{
		m_pBDANetworkProvider.Detach();
				try {m_pBDANetworkProvider.Release();}
				catch (...){}
	}

	if (m_rotEntry)
		graphTools.RemoveFromRot(m_rotEntry);
	m_piMediaControl.Release();
	m_piGraphBuilder.Release();

}

HRESULT	BDAChannelScan::ConnectGraph()
{
	HRESULT hr = S_OK;

	m_pBDACard->Connect(m_pBDANetworkProvider);

	CComPtr <IPin> pCapturePin;
	m_pBDACard->GetCapturePin(&pCapturePin.p);

	CComPtr <IPin> pDemuxPin;
	if (FAILED(hr = graphTools.FindPin(m_pBDAMpeg2Demux, L"MPEG-2 Stream", &pDemuxPin.p, REQUESTED_PINDIR_INPUT)))
	{
		return (log << "Failed to get input pin on Demux\n").Show(hr);
	}
	
	if (FAILED(hr = m_piGraphBuilder->ConnectDirect(pCapturePin, pDemuxPin, NULL)))
	{
		return (log << "Failed to connect Capture filter to BDA Demux\n").Show(hr);
	}

	if (FAILED(hr = graphTools.ConnectFilters(m_piGraphBuilder, m_pBDAMpeg2Demux, m_pBDATIF)))
	{
		return (log << "Failed to connect to BDA Demux to TIF\n").Show(hr);
	}

	if (FAILED(hr = graphTools.ConnectFilters(m_piGraphBuilder, m_pBDAMpeg2Demux, m_pBDASecTab)))
	{
		return (log << "Failed to connect BDA Demux to MPEG-2 Sections and Tables\n").Show(hr);
	}

	return hr;
}

HRESULT	BDAChannelScan::CreateGraph()
{
	HRESULT hr = S_OK;

	if(FAILED(hr = m_piGraphBuilder.CoCreateInstance(CLSID_FilterGraph)))
	{
		return (log << "Failed to create an instance of the graph builder\n").Show(hr);
	}
	if(FAILED(hr = m_piGraphBuilder->QueryInterface(IID_IMediaControl, reinterpret_cast<void **>(&m_piMediaControl.p))))
	{
		return (log << "Failed to get Media Control interface\n").Show(hr);
	}

	if(FAILED(hr = graphTools.AddToRot(m_piGraphBuilder, &m_rotEntry)))
	{
		return (log << "Failed adding to ROT\n").Show(hr);
	}
	
	return S_OK;
}

HRESULT	BDAChannelScan::BuildGraph()
{
	HRESULT hr = S_OK;

    CComBSTR bstrNetworkType;
    CLSID CLSIDNetworkType;
	LONG frequency = 0;
	LONG bandwidth = 0;

	// Initialise the tune request
	if (FAILED(hr = graphTools.InitDVBTTuningSpace(m_pTuningSpace)))
	{
		return (log << "Failed to initialise the Tune Request\n").Show(hr);
	}

	// Get the current Network Type clsid
    if (FAILED(hr = m_pTuningSpace->get_NetworkType(&bstrNetworkType)))
	{
		return (log << "Failed to get TuningSpace Network Type\n").Show(hr);
    }

	if (FAILED(hr = CLSIDFromString(bstrNetworkType, &CLSIDNetworkType)))
	{
		return (log << "Couldn't get CLSIDFromString\n").Show(hr);
	}

    // create the network provider based on the clsid obtained from the tuning space
	if (FAILED(hr = graphTools.AddFilter(m_piGraphBuilder, CLSIDNetworkType, &m_pBDANetworkProvider, L"Network Provider")))
	{
		return (log << "Failed to add Network Provider to the graph\n").Show(hr);
	}

	//Create TuneRequest
	CComPtr <ITuneRequest> pTuneRequest;
	if (FAILED(hr = graphTools.CreateDVBTTuneRequest(m_pTuningSpace, pTuneRequest, frequency, bandwidth)))
	{
		return (log << "Failed to create the Tune Request.\n").Show(hr);
	}

	//Apply TuneRequest
	if (FAILED(hr = m_pBDANetworkProvider->QueryInterface(__uuidof(IScanningTuner), reinterpret_cast<void **>(&m_piTuner))))
	{
		return (log << "Failed while interfacing Tuner with Network Provider\n").Show(hr);
	}
	if (FAILED(m_piTuner->put_TuningSpace(m_pTuningSpace)))
	{
		return (log << "Failed to give tuning space to tuner.\n").Show(hr);
	}
	if (FAILED(hr = m_piTuner->put_TuneRequest(pTuneRequest)))
	{
		return (log << "Failed to submit the Tune Tequest to the Network Provider\n").Show(hr);
	}

	//We can now add the rest of the source filters

	if (FAILED(hr = m_pBDACard->AddFilters(m_piGraphBuilder)))
	{
		return hr;
	}
	
	if (FAILED(hr = graphTools.AddFilter(m_piGraphBuilder, CLSID_MPEG2Demultiplexer, &m_pBDAMpeg2Demux, L"BDA MPEG-2 Demultiplexer")))
	{
		return (log << "Failed to add BDA MPEG-2 Demultiplexer to the graph. This may indicate that the Microsoft BDA components are not correctly installed. Type:\n  regsvr32 mpg2splt.ax\n  regsvr32 psisrndr.ax\n  regsvr32 mpeg2data.ax\n").Show(hr);
	}

	if (FAILED(hr = graphTools.AddFilterByName(m_piGraphBuilder, &m_pBDATIF, KSCATEGORY_BDA_TRANSPORT_INFORMATION, L"BDA MPEG2 Transport Information Filter")))
	{
		return (log << "Cannot load TIF. This may indicate that the Microsoft BDA components are not correctly installed. Type:\n  regsvr32 mpg2splt.ax\n  regsvr32 psisrndr.ax\n  regsvr32 mpeg2data.ax\n\n").Show(hr);
	}

	if (FAILED(hr = graphTools.AddFilterByName(m_piGraphBuilder, &m_pBDASecTab, KSCATEGORY_BDA_TRANSPORT_INFORMATION, L"MPEG-2 Sections and Tables")))
	{
		return (log << "Cannot load MPEG-2 Sections and Tables. This may indicate that the Microsoft BDA components are not correctly installed. Type:\n  regsvr32 mpg2splt.ax\n  regsvr32 psisrndr.ax\n  regsvr32 mpeg2data.ax\n\n").Show(hr);
	}

	return S_OK;
}

HRESULT BDAChannelScan::createConnectionPoint()
{
	HRESULT hr;

	/*if (FAILED(hr = m_pBDATIF->QueryInterface(__uuidof(IGuideData), reinterpret_cast<void **>(&m_piGuideData))))
	{
		return (log << "Failed to interface GuideData with TIF\n").Show(hr);
	}*/

	CComPtr <IConnectionPointContainer> piContainer;
	if (FAILED(hr = m_pBDATIF->QueryInterface(__uuidof(IConnectionPointContainer), reinterpret_cast<void **>(&piContainer))))
	{
		return (log << "Failed to interface ConnectionPoint with TIF\n").Show(hr);
	}
	if (FAILED(hr = piContainer->FindConnectionPoint(__uuidof(IGuideDataEvent), &m_piConnectionPoint)))
	{
		return (log << "Failed to find connection point on TIF\n").Show(hr);
	}

	if (FAILED(hr = m_piConnectionPoint->Advise(this, &m_dwAdviseCookie)))
	{
		m_piConnectionPoint.Release();
		return (log << "Failed to Advice connection point of TIF\n").Show(hr);
	}
	
	return S_OK;
}

STDMETHODIMP_(ULONG) BDAChannelScan::AddRef(void)
{
	return ++m_ulRef; // NOT thread-safe
}

STDMETHODIMP_(ULONG) BDAChannelScan::Release(void)
{
	--m_ulRef; // NOT thread-safe
	if (m_ulRef == 0)
	{
		delete this;
		return 0; // Can't return the member of a deleted object.
	}
	else
	{
		return m_ulRef;
	}
}

STDMETHODIMP BDAChannelScan::QueryInterface(REFIID iid, void **ppv)
{
	*ppv = NULL;
	if (iid == IID_IUnknown || iid == IID_IGuideDataEvent)
	{
		*ppv = static_cast<IGuideDataEvent *>(this);
	}

	if (*ppv)
	{
		AddRef();
		return S_OK;
	}
	else
		return E_NOINTERFACE;
}

STDMETHODIMP BDAChannelScan::ServiceChanged(VARIANT varServiceDescriptionID)
{
	DWORD dwWaitResult; 
    dwWaitResult = WaitForSingleObject(m_hServiceChangedMutex, 1000);
	if (dwWaitResult == WAIT_OBJECT_0)
	{
		try
		{
			if (m_bScanning)
				m_mpeg2parser.StartMpeg2DataScan();
		}
		catch(...)
		{
		} 
		ReleaseMutex(m_hServiceChangedMutex);
	}

	return S_OK;
}

STDMETHODIMP BDAChannelScan::ProgramChanged(VARIANT varProgramDescriptionID)
{
	return S_OK;
}
STDMETHODIMP BDAChannelScan::ScheduleEntryChanged(VARIANT varScheduleEntryDescriptionID)
{
	return S_OK;
}
STDMETHODIMP BDAChannelScan::GuideDataAcquired()
{
	return S_OK;
}
STDMETHODIMP BDAChannelScan::ProgramDeleted(VARIANT stVariant)
{
	return S_OK;
}
STDMETHODIMP BDAChannelScan::ServiceDeleted(VARIANT stVariant)
{
	return S_OK;
}
STDMETHODIMP BDAChannelScan::ScheduleDeleted(VARIANT stVariant)
{
	return S_OK;
}

BOOL BDAChannelScan::StartGraph()
{
	m_piMediaControl->Run();

	return TRUE;
}

BOOL BDAChannelScan::StopGraph()
{
	if (m_piMediaControl)
		m_piMediaControl->Stop();

	return TRUE;
}

void BDAChannelScan::VerboseToFileON()
{
	if(!m_bVerbose)
	{
		m_logfileVerboseHandle = m_mpeg2parser.VerboseOutput()->AddCallback(&lmw);
	}
}

void BDAChannelScan::VerboseToFileOFF()
{
	if(!m_bVerbose)
	{
		m_mpeg2parser.VerboseOutput()->RemoveCallback(m_logfileVerboseHandle);
	}
}

void BDAChannelScan::ToggleVerbose()
{
	m_bVerbose = !m_bVerbose;
	if (m_bVerbose)
	{
		m_consoleVerboseHandle = m_mpeg2parser.VerboseOutput()->AddCallback(&m_console);
		m_logfileVerboseHandle = m_mpeg2parser.VerboseOutput()->AddCallback(&lmw);
	}
	else
	{
		m_mpeg2parser.VerboseOutput()->RemoveCallback(m_consoleVerboseHandle);
		m_mpeg2parser.VerboseOutput()->RemoveCallback(m_logfileVerboseHandle);
	}
}

void BDAChannelScan::ToggleLockDetectionMode()
{
	m_nLockDetectionMode++;
	if (m_nLockDetectionMode >= LOCK_MODE_COUNT)
		m_nLockDetectionMode = 0;
}







/*
=======================================================================================================
=======================================================================================================
=======================================================================================================

Scan All Frequencies

=======================================================================================================
=======================================================================================================
=======================================================================================================
*/



//  Procedure to scan all UK frequencies, without any offset search
HRESULT BDAChannelScan::scanAllUK()
{
	HRESULT hr = S_OK;

/*	HANDLE hInput = ::GetStdHandle(STD_INPUT_HANDLE);
	INPUT_RECORD irInBuf[1];
	DWORD numRead = 0;
*/
	static int ulFrequencyArray[48] =
	{
		// UK UHF
		474000,  // 21
		482000,  // 22
		490000,  // 23
		498000,  // 24
		506000,  // 25
		514000,  // 26
		522000,  // 27
		530000,  // 28
		538000,  // 29
		546000,  // 30
		554000,  // 31
		562000,  // 32
		570000,  // 33
		578000,  // 34
		586000,  // 35
		594000,  // 36
		602000,  // 37
		610000,  // 38
		618000,  // 39
		626000,  // 40
		634000,  // 41
		642000,  // 42
		650000,  // 43
		658000,  // 44
		666000,  // 45
		674000,  // 46
		682000,  // 47
		690000,  // 48
		698000,  // 49
		706000,  // 50
		714000,  // 51
		722000,  // 52
		730000,  // 53
		738000,  // 54
		746000,  // 55
		754000,  // 56
		762000,  // 57
		770000,  // 58
		778000,  // 59
		786000,  // 60
		794000,  // 61
		802000,  // 62
		810000,  // 63
		818000,  // 64
		826000,  // 65
		834000,  // 66
		842000,  // 67
		850000   // 68
	};

	if (FAILED(hr = SetupGraph()))
		return hr;

/*for summary, remove for now
	summ=0;
*/
	(log << "\n").Write();
	int chNum = 0;
	for (int count=0 ; count<48 ; count++ )
	{
/*		//allow user to cancel
		if (::WaitForSingleObject(hInput, 100) == WAIT_OBJECT_0)
		{
			ReadConsoleInput(hInput, irInBuf, 1, &numRead);
			if ((numRead == 1) && (irInBuf[0].EventType == KEY_EVENT)) // make sure it's a key event, not a mouse or focus event
				count = 48;
		}
*/		
		chNum++;
		(log << "# " << ulFrequencyArray[count] << "kHz\n").Write();
		hr = scanChannel(chNum, ulFrequencyArray[count], 8);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;
		(log << "# Nothing found on " << ulFrequencyArray[count] << "kHz \n").Show();
		(log << "\n").Write();

		chNum--;
	}
/*=======================================================
remove this for now
//=========================================================
//code for summary

	//Print summary to console
	cout << "Summary" << endl;
	for (summ=0; summ<N_SUMMARY; summ++)
		printsummary (summary[summ]);
	return 0;
//====================================================
*/
	DestroyGraph();
	return hr;
}

//  Procedure to scan all UK frequencies, with an automatic 166kHz offset search
HRESULT BDAChannelScan::scanAllUK166()
{
	HRESULT hr = S_OK;

	static int ulFrequencyArray[] =
	{
		// UK UHF
		306000,
		462000,
		474000,  // 21
		482000,  // 22
		490000,  // 23
		498000,  // 24
		506000,  // 25
		514000,  // 26
		522000,  // 27
		530000,  // 28
		538000,  // 29
		546000,  // 30
		554000,  // 31
		562000,  // 32
		570000,  // 33
		578000,  // 34
		586000,  // 35
		594000,  // 36
		602000,  // 37
		610000,  // 38
		618000,  // 39
		626000,  // 40
		634000,  // 41
		642000,  // 42
		650000,  // 43
		658000,  // 44
		666000,  // 45
		674000,  // 46
		682000,  // 47
		690000,  // 48
		698000,  // 49
		706000,  // 50
		714000,  // 51
		722000,  // 52
		730000,  // 53
		738000,  // 54
		746000,  // 55
		754000,  // 56
		762000,  // 57
		770000,  // 58
		778000,  // 59
		786000,  // 60
		794000,  // 61
		802000,  // 62
		810000,  // 63
		818000,  // 64
		826000,  // 65
		834000,  // 66
		842000,  // 67
		850000   // 68
	};

  m_ChannelList.reset();

	if (FAILED(hr = SetupGraph()))
		return hr;

  int chCount = sizeof(ulFrequencyArray)/sizeof(ulFrequencyArray[0]);
  summ_Count = 0;
	int chNum = 0;
	for (int count=0 ; count<chCount ; count++ )
	{
		chNum++;
		(log << "# " << ulFrequencyArray[count] << "kHz\n").Write();
		if(count >= 2)
		{
			(log << "# Channel " << (19+count) << "\n").Write();
			cout << "# Channel " << (19+count) << "\n";
		}
		hr = scanChannel(chNum, ulFrequencyArray[count], 8);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << "# Nothing found on " << ulFrequencyArray[count] << "kHz, +166kHz";
		(log << "# trying +166kHz\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] + 166, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

/*		cout << ", +167kHz";
		(log << "# trying +167kHz\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] + 167, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;
*/
		cout << ", -166kHz";
		(log << "# trying -166kHz\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] - 166, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

/*		cout << ", -167kHz";
		(log << "# trying -167kHz\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] - 167, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;
*/
		(log << "# Nothing found on " << ulFrequencyArray[count] << "kHz\n\n").Write();
		cout << ", Nothing found\n";


		chNum--;
	}

	(log << "Summary of frequencies\n").Write();
	(log << "======================\n\n").Write();


	for (int countsumm=0 ; countsumm<summ_Count ; countsumm++)
	{
		(log << "Frequency=").Write();
		(log << summ_freq[countsumm]).Write();
		(log << "kHz  Strength=").Write();
		(log << summ_strength[countsumm]).Write();
		(log << "  Quality=").Write();
		(log << summ_quality[countsumm]).Write();
		(log << "%").Write();
		//(log << summ_transmitter[countsumm]).Write();
		(log << "\n").Write();
	}

	(log << "\nPlease note that strength may not be an accurate figure.  The actual meaning\n").Write();
	(log << "of strength depends upon the manufacturer of your device.  For Hauppauge cards\n").Write();
	(log << "and Philips SAA71xx hybrid boards please see\n").Write();
	(log << "http://www.quantexzone.com/faq/hcwfaq/wintv_bda_signalstats.htm\n").Write();

	setrgb(0);
	DestroyGraph();

	return hr;
}

//  Procedure to scan all Australian frequencies
HRESULT BDAChannelScan::scanAllAus()
{
	HRESULT hr = S_OK;

	static int ulFrequencyArray[50] =
	{
		// Band III VHF
		177500,  // 6
		184500,  // 7
		191500,  // 8
		198500,  // 9
		205500,  // 9A
		212500,  // 10
		219500,  // 11
		226500,  // 12

		// Band IV UHF
		529500,  // 28
		536500,  // 29
		543500,  // 30
		550500,  // 31
		557500,  // 32
		564500,  // 33
		571500,  // 34
		578500,  // 35

		// Band V UHF
		585500,  // 36
		592500,  // 37
		599500,  // 38
		606500,  // 39
		613500,  // 40
		620500,  // 41
		627500,  // 42
		634500,  // 43
		641500,  // 44
		648500,  // 45
		655500,  // 46
		662500,  // 47
		669500,  // 48
		676500,  // 49
		683500,  // 50
		690500,  // 51
		697500,  // 52
		704500,  // 53
		711500,  // 54
		718500,  // 55
		725500,  // 56
		732500,  // 57
		739500,  // 58
		746500,  // 59
		753500,  // 60
		760500,  // 61
		767500,  // 62
		774500,  // 63
		781500,  // 64
		788500,  // 65
		795500,  // 66
		802500,  // 67
		809500,  // 68
		816500   // 69
	};

  m_ChannelList.reset();

	if (FAILED(hr = SetupGraph()))
		return hr;

	int chNum = 0;
	for (int count=0 ; count<50 ; count++ )
	{
		chNum++;
		hr = scanChannel(chNum, ulFrequencyArray[count], 7);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << "# Nothing found on " << ulFrequencyArray[count] << "kHz, trying +125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] + 125, 7);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", trying -125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNum, ulFrequencyArray[count] - 125, 7);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", Nothing found\n").Show();
		(log << "\n").Write();
		
		chNum--;
	}

	setrgb(0);
	DestroyGraph();

	return hr;
}

//  Procedure to scan all European frequencies
HRESULT BDAChannelScan::scanAllEuro()
{
	HRESULT hr = S_OK;

	// VHF 7MHz
	static int ulFrequencyArrayVHF7[15] =
	{
		// VHF 7Mhz
		84500,
		56000,
		64500,
		177500,
		184500,
		186000,
		194500,
		191500,
		191625,
		198500,
		203500,
		205500,
		212500,
		219500,
		226500
	};

  m_ChannelList.reset();

	if (FAILED(hr = SetupGraph()))
		return hr;

	int chNum = 0;
	for (int countVHF7=0 ; countVHF7<15 ; countVHF7++ )
	{
		cout << "# " << ulFrequencyArrayVHF7[countVHF7] << "kHz";
		(log << "# Scanning " << ulFrequencyArrayVHF7[countVHF7] << "kHz, 7MHz bandwidth\n").Write();
		chNum++;
		hr = scanChannel(chNum, ulFrequencyArrayVHF7[countVHF7], 7);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << "# Nothing found on " << ulFrequencyArrayVHF7[countVHF7] << "kHz, trying +125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNum, ulFrequencyArrayVHF7[countVHF7] + 125, 7);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", trying -125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNum, ulFrequencyArrayVHF7[countVHF7] - 125, 7);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", Nothing found\n").Show();
		(log << "\n").Write();

		chNum--;
	}
	DestroyGraph();

	// VHF 8MHz
	static int ulFrequencyArrayVHF8[15] =
	{
		// VHF 8Mhz
		84500,
		56000,
		64500,
		177500,
		184500,
		186000,
		194500,
		191500,
		191625,
		198500,
		203500,
		205500,
		212500,
		219500,
		226500
	};

	if (FAILED(hr = SetupGraph()))
		return hr;

	int chNumVHF8 = 0;
	for (int countVHF8=0 ; countVHF8<15 ; countVHF8++ )
	{
		(log << "# Scanning " << ulFrequencyArrayVHF8[countVHF8] << "kHz, 8MHz bandwidth \n").Write();
		cout << "# " << ulFrequencyArrayVHF8[countVHF8] << "kHz";
		chNumVHF8++;
		hr = scanChannel(chNumVHF8, ulFrequencyArrayVHF8[countVHF8], 8);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << "# Nothing found on " << ulFrequencyArrayVHF8[countVHF8] << "kHz, trying +125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNumVHF8, ulFrequencyArrayVHF8[countVHF8] + 125, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", trying -125").Show();
		(log << "\n").Write();

		hr = scanChannel(chNumVHF8, ulFrequencyArrayVHF8[countVHF8] - 125, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", Nothing found\n").Show();
		(log << "\n").Write();

		chNumVHF8--;
	}
	DestroyGraph();
  
  static int ulFrequencyArray[53] =
  {
		// UHF 8MHz
		306000,
		462000,
		474000,  // 21
		482000,  // 22
		490000,  // 23
		498000,  // 24
		506000,  // 25
		514000,  // 26
		522000,  // 27
		530000,  // 28
		538000,  // 29
		546000,  // 30
		554000,  // 31
		562000,  // 32
		570000,  // 33
		578000,  // 34
		586000,  // 35
		594000,  // 36
		602000,  // 37
		610000,  // 38
		618000,  // 39
		626000,  // 40
		634000,  // 41
		642000,  // 42
		650000,  // 43
		658000,  // 44
		666000,  // 45
		674000,  // 46
		682000,  // 47
		690000,  // 48
		698000,  // 49
		706000,  // 50
		714000,  // 51
		722000,  // 52
		730000,  // 53
		738000,  // 54
		746000,  // 55
		754000,  // 56
		762000,  // 57
		770000,  // 58
		778000,  // 59
		786000,  // 60
		794000,  // 61
		802000,  // 62
		810000,  // 63
		818000,  // 64
		826000,  // 65
		834000,  // 66
		842000,  // 67
		850000,  // 68
		858000,  // 69
		816500,
		866000
	};

	if (FAILED(hr = SetupGraph()))
		return hr;
	int chNumUHF  = 0;
	for (int count=0 ; count<53 ; count++ )
	{
		(log << "# Scanning " << ulFrequencyArray[count] << "kHz, 8MHz bandwidth \n").Write();
		cout << "# " << ulFrequencyArray[count] << "kHz";
		chNumUHF++;
		hr = scanChannel(chNumUHF, ulFrequencyArray[count], 8);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", +125kHz";
		(log << "# trying +125kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] + 125, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", +166kHz";
		(log << "# trying +166kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] + 166, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", +167kHz";
		(log << "# trying +167kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] + 167, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", -125kHz";
		(log << "# trying -125kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] - 125, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", -166kHz";
		(log << "# trying -166kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] - 166, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		cout << ", -167kHz";
		(log << "# trying -167kHz\n").Write();

		hr = scanChannel(chNumUHF, ulFrequencyArray[count] - 167, 8);
		if (hr == E_FAIL)
		{
			return (log << "\nError locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << ", Nothing found\n").Show();
		(log << "\n").Write();

		chNumUHF--;
	}

	DestroyGraph();

	return hr;
}

//  Procedure to scan all Taiwanese frequencies
HRESULT BDAChannelScan::scanAllTaiw()
{
	HRESULT hr = S_OK;

	static int ulFrequencyArray[10] =
	{
		533000,
		539000,
		545000,
		551000,
		557000,
		563000,
		575000,
		581000,
		593000,
		599000
  };

  m_ChannelList.reset();

	if (FAILED(hr = SetupGraph()))
		return hr;

	int chNum = 0;
	for (int count=0 ; count<10 ; count++ )
	{
		chNum++;
		hr = scanChannel(chNum, ulFrequencyArray[count], 6);
		if (hr == E_FAIL)
		{
			return (log << "Error locking channel. Aborting\n").Show(E_FAIL);
		}
		if (hr == S_OK)
			continue;

		(log << "# Nothing found on " << ulFrequencyArray[count] << "kHz").Show();
		(log << "\n").Write();
		chNum--;
	}

	DestroyGraph();
	return hr;
}


// lock channel for multi-frequency sweep scan
HRESULT BDAChannelScan::scanChannel(long channelNumber, long frequency, long bandwidth)
{
	ResetEvent(m_hGuideDataChangedEvent);
	m_mpeg2parser.Reset();
	m_mpeg2parser.SetFilter(m_pBDASecTab);
	m_mpeg2parser.SetNetworkNumber(channelNumber);
	
	m_bScanning = TRUE;

	BOOL bLocked = FALSE;
	BOOL bPresent = FALSE;
	
	//reset all statistics
	long nStrength = 0;
	long nQuality = 0;
	long nSampleTime = 0;
	long QualityMin = 0;
	long QualityMax = 0;
	long QualityAveCount = 0;
	long QualityAveTotal = 0;
	long QualityAve = 0;
	long StrengthAveCount = 0;
	long StrengthAveTotal = 0;
	long StrengthAve = 0;
	int scancount = 0;
	int statcount = 0;
  transponder* ct = NULL;

	//attempt to lock channel
	HRESULT hr = LockChannel(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);

	//if locked, get information
	switch (hr)
	{
		case S_OK:

			cout << "\n\n\n============================================================================";
			cout << "\n# locked on " << frequency <<  "KHz, please wait for network information..." << "\n";

      m_mpeg2parser.StartMpeg2DataScan();
			switch (m_mpeg2parser.WaitForScanToFinish())
			{
			case WAIT_OBJECT_0:
			
				(log << "\n\n============================================================================\n\n").Write();
				(log << "Frequency (Scanned) = " << frequency << "kHz\n").Write();

				cout << "\n# Network Information...\n\n";
        m_mpeg2parser.PrintDigitalWatchChannelsIni();
				
				VerboseToFileON();
        m_mpeg2parser.PrintDigitalWatchChannelsFull();
				VerboseToFileOFF();
        // populate the local list of channels
        ct = m_mpeg2parser.GetCurrentTransponder();
        if(ct)
        {
          Channel new_channel;
          // Fill in the details for this transponder
          new_channel.m_TransportID = ct->transport_stream_id;
          new_channel.m_NetworkID = ct->original_network_id;
          new_channel.m_Frequency = m_bSageTVUseExactFrequency ? ct->frequency : frequency;
          new_channel.m_Band = ct->bandwidth;
          // now each channel on the transponder
          vector<service *>::iterator it;
          for ( it = ct->services.begin() ; it != ct->services.end() ; it++ )
          {
            memset(new_channel.m_Name, ' ', 128);
            memset(new_channel.m_Provider, ' ', 128);
            if((*it)->service_name)
            {
              strncpy(new_channel.m_Name, (char*)((*it)->service_name), 128);
            }
             if(new_channel.m_Name[0] == ' ')
              new_channel.m_Name[0] = '\0';
            if((*it)->provider_name)
            {
              strncpy(new_channel.m_Provider, (char*)((*it)->provider_name), 128);
            }
            if(new_channel.m_Provider[0] == ' ')
              new_channel.m_Provider[0] = '\0';

            bool add_to_list = false;
            new_channel.m_Type = (*it)->type;
            switch((*it)->type)
            {
            case 1: // TV channel
              //(log << "* TV station found = '" << new_channel.m_Name << "[" << (*it)->channel_num << "]'\n").Write();
              if((*it)->service_name && (*it)->channel_num != 0)
              {
                add_to_list = true;
              }
              break;
            case 2: // Radio channel
              //(log << "* Radio station found = '" << new_channel.m_Name << "[" << (*it)->channel_num << "]'\n").Write();
              if((*it)->service_name && (*it)->channel_num != 0)
              {
                add_to_list = true;
              }
              break;
            default:  // other channel type
              //(log << "* Other station found = '" << new_channel.m_Name << "[" << (*it)->channel_num << "]'\n").Write();
              break;
            }

            if(add_to_list)
            {
              // Populate the channel details
              strncpy(new_channel.m_Name, (char*)((*it)->service_name), 128);
              new_channel.m_ServiceID = (*it)->service_id;
              new_channel.m_CtrlBits = (*it)->scrambled ? 3 : 1;
              new_channel.m_OrigLocation = new_channel.m_Location = (*it)->channel_num;
							new_channel.m_SummaryIDX = summ_Count;

              // record the channel details
              // new_channel.print(log);

              // add the channel to the list
              m_ChannelList.add(new_channel);
            }
          }
        }

				cout << "\n# Signal Statistics...\n";

				for (statcount = 0; statcount < STATCOUNT; statcount++)
				{
					Sleep(STATTIME);
					cout << ".";
				}
				cout << "\n";

				for (scancount = 0; scancount<SCANCOUNT; scancount++)
				{
					if (FAILED(hr = GetSignalStatistics(bLocked, bPresent, nStrength, nQuality, nSampleTime)))
						return hr;
					
					QualityAveCount = QualityAveCount + 1;
					StrengthAveCount = StrengthAveCount + 1;
					QualityAveTotal = QualityAveTotal + nQuality;
					StrengthAveTotal = StrengthAveTotal + nStrength;
					(log << "# Locked = " << (bLocked ? "Y" : "N") << "    Present = " << (bPresent ? "Y" : "N") << "    Strength = " << nStrength << "    Quality = " << nQuality << "%\n").Show();
					//(log << "\n# locked = " << (bLocked ? "Y" : "N") << ", present = " << (bPresent ? "Y" : "N") << ", strength = " << nStrength << ", quality = " << nQuality << "%").Show() ;
					Sleep(SCANTIME);
				}
				QualityAve = QualityAveTotal / QualityAveCount;
				StrengthAve = StrengthAveTotal / StrengthAveCount;
				(log << "\n# Average quality on " << frequency << "kHz = " << QualityAve << "%\n").Show();
/*=========================================================
//=========================================================
remove this for now
//=========================================================
//code for summary

		//	Collect data for the summary
				summary[summ].summary_freq = frequency;
				summary[summ].summary_str = nStrength;
				summary[summ].summary_qual = QualityAve;
		//		summary[summ].summary_mux = "";
		//		summary[summ].summary_trans = m_mpeg2parser.current_tp->network_name;
				summ++;
//=========================================================
//========================================================*/


	if (summ_Count < 256)
	{
		summ_freq[summ_Count] = frequency;
		summ_strength[summ_Count] = StrengthAve;
		summ_quality[summ_Count] = QualityAve;
		summ_transmitter[summ_Count] = m_mpeg2parser.TransmitterName();
		//summ_transmitter[summ_Count] = m_mpeg2parser.current_tp->network_name;
		//summ_streamid[summ_Count] = streamid;
		summ_Count++;
	}
	else
	{
		//exceeded the 256 limit
	}

				(log << "\n============================================================================\n\n\n").Show();
				break;
			case WAIT_TIMEOUT:
				printf("# scan timed out\n\n");
				(log << "# scan timed out\n\n").Write();
				break;
			default:
				printf("# scan failed\n\n");
				(log << "# scan failed\n\n").Write();
				break;
			};
			break;

		default:
			break;
	}
	m_mpeg2parser.ReleaseFilter();
	m_bScanning = FALSE;
	return hr;
}

#include <algorithm>

void BDAChannelScan::ShowChannelResults()
{
	if(!m_bSageTVOutputChannelList)
	{
		return;
	}

	vector<Channel>::iterator cit = m_ChannelList.begin();

#if 0
	//DBG - test code to throw in some random duplicates
	int last_lcn = 999;
	while(cit != m_ChannelList.end())
	{
		if(rand()%100 > 50)
		{
			if(last_lcn == 999)
			{
				last_lcn = cit->m_Location;
			}
			else
			{
				int tmp = cit->m_Location;
				cit->m_Location = cit->m_OrigLocation = last_lcn;
				if(rand()%100 > 50)
				{
					last_lcn = tmp;
				}
			}
		}
		cit++;
	}
	cit = m_ChannelList.begin();
	//DBG
#endif

	// Scan order or LCN order?
	if(m_bSageTVSortByLCN)
	{
		// Experimental code to move channels with duplicate LCN's to channel 800+.
		//	The channel with the strongest signal should take the actual LCN and all others
		//	should be moved.
		if(m_bSageTVMoveDuplicates)
		{
			// Walk through the channel list and find any duplicate channels. These 
			//	duplicates need to be stored in the homeless channels area (channel 800+)
			int duplicate_channel_num = 800;	// next place to store channel with a duplicate LCN
			cit = m_ChannelList.begin();
			while(cit != m_ChannelList.end())
			{
				// find a channel with a duplicate LCN
				vector<Channel>::iterator dup = cit;
				while(dup != m_ChannelList.end())
				{
					if(dup != cit &&											// don't compare with itself
						dup->m_Location == cit->m_Location)	// same LCN?
					{
						break;
					}
					dup++;
				}

				// duplicate found?
				if(dup != m_ChannelList.end())
				{
					// the channel with the lowest signal strength gets moved to the duplicate area
					if(summ_strength[dup->m_SummaryIDX] <= summ_strength[cit->m_SummaryIDX])
					{
						dup->m_Location = duplicate_channel_num;
					}
					else
					{
						cit->m_Location = duplicate_channel_num;
					}
					duplicate_channel_num++;
					// start from the beginning of the list again
					cit = m_ChannelList.begin();
				}
				else
				{
					// move to next channel if we didn't find a match
					cit++;
				}
			}
		}

		// sort the final list
		std::sort(m_ChannelList.begin(), m_ChannelList.end());
	}

	(log << "\nSummary of TV Channels\n").Write();
	(log << "======================\n\n").Write();

  int i=0;	// channel list index
  cit = m_ChannelList.begin();
  while(cit != m_ChannelList.end())
  {
    if(cit->m_Type == 1 ||													// include tv stations
      (cit->m_Type == 2 && m_bSageTVIncludeRadio))	// include radio stations
    {
		i++;	// keep track of output channel list index
		(log << "CH:" << (m_bSageTVSortByLCN ? cit->m_Location : i) <<
        " onid:" << cit->m_NetworkID <<
        " tsid:" << cit->m_TransportID <<
        " sid:" << cit->m_ServiceID <<
        " frq:" << cit->m_Frequency*1000 <<
        " band:" << cit->m_Band <<
        " ctrl:" << cit->m_CtrlBits <<
        " #:" << cit->m_Name << "\n").Write();
    }
    cit++;
  }
  (log << " " << ((int)m_ChannelList.size()) << " Channel(s) found\n\n").Write();

}

/*
=======================================================================================================
=======================================================================================================
=======================================================================================================

Single Frequency Statistics

=======================================================================================================
=======================================================================================================
=======================================================================================================
*/



//  Gets signal statistics for a single frequency
//  Averaged statistics
HRESULT BDAChannelScan::SignalStatisticsAveraged(long frequency, long bandwidth)
{
	HRESULT hr;

	if (FAILED(hr = SetupGraph()))
		return hr;

	cout << endl;
	cout << "Press any key to end statistics and return to main menu..." << endl;
	cout << endl;

	BOOL bLocked = FALSE;
	BOOL bPresent = FALSE;
	long nStrength = 0;
	long nQuality = 0;
	long nSampleTime = 0;

	long QualityMin = 0;
	long QualityMax = 0;
	long QualityAveCount = 0;
	long QualityAveTotal = 0;
	long QualityAve = 0;

	long StrengthMin = 0;
	long StrengthMax = 0;
	long StrengthAveCount = 0;
	long StrengthAveTotal = 0;
	long StrengthAve = 0;

	BOOL bKeyboardEvent = FALSE;
	HANDLE hInput = ::GetStdHandle(STD_INPUT_HANDLE);
	INPUT_RECORD irInBuf[1];
	DWORD numRead = 0;

	(log << "\n# locking " << frequency << ", " << bandwidth << "\n").Show();
	
	hr = LockChannel(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);	

	cout << endl << "Please wait..";
	for (int statcount = 0; statcount < STATCOUNT; statcount++)
	{
	Sleep(STATTIME);
	cout << ".";
	}
	cout << "\n";


//	if (bLocked = FALSE)
//	{
//		cout << "# signal locked = " << (bLocked ? "Y" : "N")
//			 << " present = " << (bPresent ? "Y" : "N")
//			 << " strength = " << nStrength
//			 << " quality = " << nQuality << "\n";
//	};

	if (bLocked == FALSE)
	{
		cout << "Unable to lock onto a usable signal on this frequency" << endl;
		if (bPresent == TRUE)
			{
			cout << "A signal was present but it is either too poor quality or is an analogue channel" << endl;
			Sleep(2000);
			}
		return hr;
	};

	SignalStatsCont(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);
	QualityMax = nQuality;
	QualityMin = nQuality;

	StrengthMax = nStrength;
	StrengthMin = nStrength;

	do
	{
		hr = SignalStatsCont(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);

		if (nQuality > QualityMax)
		{
		QualityMax = nQuality;
		}

		if (nQuality < QualityMin)
		{
		QualityMin = nQuality;
		}

		if (nStrength > StrengthMax)
		{
		StrengthMax = nStrength;
		}

		if (nStrength < StrengthMin)
		{
		StrengthMin = nStrength;
		}

		QualityAveCount = QualityAveCount + 1;
		QualityAveTotal = QualityAveTotal + nQuality;
		QualityAve = QualityAveTotal / QualityAveCount;

		StrengthAveCount = StrengthAveCount + 1;
		StrengthAveTotal = StrengthAveTotal + nStrength;
		StrengthAve = StrengthAveTotal / StrengthAveCount;

		//clrscr();
		system("CLS");
		cout << endl;
		cout << "Signal Statistics for " << frequency << " MHz" << endl;
		cout << endl;
		cout << "_______________________________________________________________________" << endl;
		cout << endl;
		cout << "    Strength = " << nStrength << endl;
		cout << endl;
		cout << endl;
		cout << "    " << StrengthMin << " (Min)  /  " << StrengthAve << " (Average)  /  " << StrengthMax << " (Max)" << endl;
		cout << endl;
		cout << "_______________________________________________________________________" << endl;
		cout << endl;
		cout << "    Quality = " << nQuality << "%" << endl;
		cout << endl;
		cout << endl;
		cout << "    0%  25%  50%  55%  60%  65%  70%  75%  80%  85%  90%  95%  100%" << endl;
		cout << "    +----+----+----+----+----+----+----+----+----+----+----+----+" << endl;
		cout << "    ";

		
		int QualBarCount = 39;

		//Set this to test the bar generation code
		//nQuality = 59;

		//quality between 50 and 69 - RED
		if (nQuality > 49)
		{
			setrgb(18);
			if (nQuality <60)
			{
			for (QualBarCount=39; QualBarCount < nQuality; QualBarCount++)
				cout << "#";
			}
		};

		//quality between 60 and 79 - YELLOW
		if (nQuality > 59)
		{
			setrgb(17);
			if (nQuality <80)
			{
			for (QualBarCount=39; QualBarCount < nQuality; QualBarCount++)
				cout << "#";
			}
		};

		//quality greater than 80 - GREEN
		if (nQuality > 79)
		{
			setrgb(16);
			for (QualBarCount=39; QualBarCount < nQuality; QualBarCount++)
				cout << "#";
		};

		if (nQuality < 5)
		{
			if (nQuality > 0)		
			{
			setrgb(18);
			cout << "#" ;
			}
		}

		if (nQuality < 10)
		{
			if (nQuality > 4)		
			{
			setrgb(18);
			cout << "##" ;
			}
		}

		if (nQuality < 15)
		{
			if (nQuality > 9)		
			{
			setrgb(18);
			cout << "###" ;
			}
		}

		if (nQuality < 20)
		{
			if (nQuality > 14)		
			{
			setrgb(18);
			cout << "####" ;
			}
		}

		if (nQuality < 25)
		{
			if (nQuality > 19)		
			{
			setrgb(18);
			cout << "#####" ;
			}
		}


		if (nQuality < 30)
		{
			if (nQuality > 24)		
			{
			setrgb(18);
			cout << "######" ;
			}
		}

		if (nQuality < 35)
		{
			if (nQuality > 29)		
			{
			setrgb(18);
			cout << "#######" ;
			}
		}

		if (nQuality < 40)
		{
			if (nQuality > 34)		
			{
			setrgb(18);
			cout << "########" ;
			}
		}

		if (nQuality < 45)
		{
			if (nQuality > 39)		
			{
			setrgb(18);
			cout << "#########" ;
			}
		}

		if (nQuality < 50)
		{
			if (nQuality > 44)		
			{
			setrgb(18);
			cout << "##########" ;
			}
		}

	setrgb(0);
		cout << endl;
		cout << "    +----+----+----+----+----+----+----+----+----+----+----+----+" << endl;
		cout << endl;
		cout << endl;
		cout << "    " << QualityMin << "% (Min)  /  " << QualityAve << "% (Average)  /  " << QualityMax << "% (Max)" << endl;
		cout << endl;
		cout << "_______________________________________________________________________" << endl;

		if (::WaitForSingleObject(hInput, 100) == WAIT_OBJECT_0)
		{
			ReadConsoleInput(hInput, irInBuf, 1, &numRead);
			if ((numRead == 1) && (irInBuf[0].EventType == KEY_EVENT)) // make sure it's a key event, not a mouse or focus event		
				bKeyboardEvent = TRUE;
		}

	} while (!bKeyboardEvent);

	DestroyGraph();
	cout << "\n";

	return hr;
}

//  Gets signal statistics for a single frequency
//  Scrolling signal statistics
HRESULT BDAChannelScan::SignalStatisticsScrolling(long frequency, long bandwidth)
{
	HRESULT hr;

	if (FAILED(hr = SetupGraph()))
		return hr;

	cout << endl;
	cout << "Press any key to end statistics and return to main menu..." << endl;
	cout << endl;

	BOOL bLocked = FALSE;
	BOOL bPresent = FALSE;
	long nStrength = 0;
	long nQuality = 0;
	long nSampleTime = 0;

	BOOL bKeyboardEvent = FALSE;
	HANDLE hInput = ::GetStdHandle(STD_INPUT_HANDLE);
	INPUT_RECORD irInBuf[1];
	DWORD numRead = 0;

	(log << "\n# locking " << frequency << ", " << bandwidth << "\n").Show();

	hr = LockChannel(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);	
		cout << "# signal locked = " << (bLocked ? "Y" : "N")
			 << " present = " << (bPresent ? "Y" : "N")
			 << " strength = " << nStrength
			 << " quality = " << nQuality << "\n";

	cout << endl << "Please wait..";
	for (int statcount = 0; statcount < STATCOUNT; statcount++)
	{
	Sleep(STATTIME);
	cout << ".";
	}
	cout << "\n\n";

	do
	{
		hr = SignalStatsCont(frequency, bandwidth, bLocked, bPresent, nStrength, nQuality, nSampleTime);
		cout << "  Locked = " << (bLocked ? "Y" : "N") << "    Present = " << (bPresent ? "Y" : "N") << "    Strength = " << nStrength << "    Quality = " << nQuality << "%\n";			
		if (::WaitForSingleObject(hInput, 1000) == WAIT_OBJECT_0)
		{
			ReadConsoleInput(hInput, irInBuf, 1, &numRead);
			if ((numRead == 1) && (irInBuf[0].EventType == KEY_EVENT)) // make sure it's a key event, not a mouse or focus event
				bKeyboardEvent = TRUE;
		}

	} while (!bKeyboardEvent);

	DestroyGraph();
	cout << "\n";

	return hr;
}

//  Routine to lock channel
//  Original
HRESULT	BDAChannelScan::LockChannel(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long&sampletime)
{
	HRESULT hr = S_OK;
	CComPtr <ITuneRequest> piTuneRequest;

	if (SUCCEEDED(hr = graphTools.CreateDVBTTuneRequest(m_pTuningSpace, piTuneRequest, lFrequency, lBandwidth)))
	{
		CComQIPtr <ITuner> pTuner(m_pBDANetworkProvider);
		if (!pTuner)
		{
			return (log << "Failed while interfacing Tuner with Network Provider\n").Show(E_FAIL);
		}
		if (FAILED(hr = pTuner->put_TuneRequest(piTuneRequest)))
		{
			return (log << "Failed to submit the Tune Tequest to the Network Provider\n").Show(hr);
		}

		pTuner.Release();
		piTuneRequest.Release();

		//Try testing the signal stats 10 times during a second to allow time for the tuner to lock.
		for (int tests = 0; tests < 10 ; tests++ )
		{
			//Check that channel locked ok.
			if (FAILED(hr = GetSignalStatistics(locked, present, strength, quality, sampletime)))
				return hr;

			(verbose << "# signal locked = " << (locked ? "Y" : "N")
					 << " present = " << (present ? "Y" : "N")
					 << " strength = " << strength
					 << " quality = " << quality << "\n").Show();
			
			//if ((locked>0) || (present>0) || (quality>0))
			switch (m_nLockDetectionMode)
			{
			case LOCK_MODE_QUALITY:
				if (quality>0)
					return S_OK;
				break;

			case LOCK_MODE_LOCKED:
			default:
				if (locked>0)
					return S_OK;
				break;
			};

			Sleep(100);
		}
		return S_FALSE;
	}

	return hr;
}

//  Routine to lock channel
//  Fast
HRESULT	BDAChannelScan::LockChannelFast(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long&sampletime)
{
	HRESULT hr = S_OK;
	CComPtr <ITuneRequest> piTuneRequest;

	if (SUCCEEDED(hr = graphTools.CreateDVBTTuneRequest(m_pTuningSpace, piTuneRequest, lFrequency, lBandwidth)))
	{
		CComQIPtr <ITuner> pTuner(m_pBDANetworkProvider);
		if (!pTuner)
		{
			return (log << "Failed while interfacing Tuner with Network Provider\n").Show(E_FAIL);
		}
		if (FAILED(hr = pTuner->put_TuneRequest(piTuneRequest)))
		{
			return (log << "Failed to submit the Tune Tequest to the Network Provider\n").Show(hr);
		}

		pTuner.Release();
		piTuneRequest.Release();

		//Try testing the signal stats 10 times during a second to allow time for the tuner to lock.
		for (int tests = 0; tests < 10 ; tests++ )
		{
			//Check that channel locked ok.
			if (FAILED(hr = GetSignalStatistics(locked, present, strength, quality, sampletime)))
				return hr;

			(verbose << "# signal locked = " << (locked ? "Y" : "N")
					 << " present = " << (present ? "Y" : "N")
					 << " strength = " << strength
					 << " quality = " << quality << "\n").Show();
			
			//if ((locked>0) || (present>0) || (quality>0))
			switch (m_nLockDetectionMode)
			{
			case LOCK_MODE_QUALITY:
				if (quality>0)
					return S_OK;
				break;

			case LOCK_MODE_LOCKED:
			default:
				if (locked>0)
					return S_OK;
				break;
			};

			Sleep(100);
		}
		return S_FALSE;
	}

	return hr;
}

//  Routine used to obtain signal statistics in loop
HRESULT BDAChannelScan::SignalStatsCont(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime)
{
	HRESULT hr = S_OK;
		for (int tests = 0; tests < 10 ; tests++ )
		{
			//Sleep(500);
			if (FAILED(hr = GetSignalStatistics(locked, present, strength, quality, sampletime)))
				return hr;
			(log << "# Locked = " << (locked ? "Y" : "N") << "    Present = " << (present ? "Y" : "N") << "    Strength = " << strength << "    Quality = " << quality << "%\n").Write();

			//if ((locked>0) || (present>0) || (quality>0))
			switch (m_nLockDetectionMode)
			{
			case LOCK_MODE_QUALITY:
				if (quality>0)
					return S_OK;
				break;

			case LOCK_MODE_LOCKED:
			default:
				if (locked>0)
					return S_OK;
				break;
			};

			Sleep(100);
		}
		return S_FALSE;
}

//  Routine to get signal statistics
HRESULT BDAChannelScan::GetSignalStatistics(BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime)
{
	return m_pBDACard->GetSignalStatistics(locked, present, strength, quality, sampletime);
}


void BDAChannelScan::AddSummary(long freq, long strength, long quality, unsigned char * transmitter, long streamid)
{
	if (summ_Count < 256)
	{
		summ_freq[summ_Count] = freq;
		summ_strength[summ_Count] = strength;
		summ_quality[summ_Count] = quality;
		summ_transmitter[summ_Count] = transmitter;
		summ_streamid[summ_Count] = streamid;
		summ_Count++;
	}
	else
	{
		//exceeded the 256 limit
	}
}
