/**
 *	ScanChannelsBDA.h
 *	Copyright (C) 2004 BionicDonkey
 *	Copyright (C) 2004 Nate
 *  Copyright (C) 2004 JoeyBloggs
 *
 *	This file is part of DigitalWatch, a free DTV watching and recording
 *	program for the VisionPlus DVB-T.
 *
 *	DigitalWatch is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	DigitalWatch is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with DigitalWatch; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef BDASCANCHANNELS_H
#define BDASCANCHANNELS_H

#include "stdafx.h"
#include "Mpeg2DataParser.h"
#include "BDACardCollection.h"
#include "FilterGraphTools.h"
#include "LogMessage.h"
#include "LogMessageConsoleOutput.h"

#include <bdatif.h>

class Channel
{
public:
  int m_Location;       // CH:
  int m_NetworkID;      // onid:
  int m_TransportID;    // tsid:
  int m_ServiceID;      // sid:
  int m_Frequency;      // frq:
  int m_Band;           // band:
  int m_CtrlBits;       // ctrl: (0:disable, 1:enable, 3:encrypted)
  int m_Type;           // 1:TV 2:Radio
  int m_OrigLocation;   // Original CH: location (can be different from m_Location if multiple channels have same location assignment i.e. may be in an area which receives reception from more than one transmitter)
	int m_SummaryIDX;			// index into the summary table (used to obtain signal strength for sorting)
  char m_Name[128];     // #: (channel name)
  char m_Provider[128]; // Provider (for intelligent sort)

  bool operator <(const Channel& other)
  {
    return m_Location < other.m_Location;
  }

  void print(LogMessage& log)
  {
    (log << " " << m_Location << " '" << m_Name << "'\n").Write();
    (log << "   onid:" << m_NetworkID << "\n").Write();
    (log << "   tsid:" << m_TransportID << "\n").Write();
    (log << "    sid:" << m_ServiceID << "\n").Write();
    (log << "   freq:" << m_Frequency << "\n").Write();
    (log << "   band:" << m_Band << "\n").Write();
    (log << "   ctrl:" << m_CtrlBits << "\n").Write();
    (log << "   type:" << m_Type << "\n").Write();
    (log << "   olcn:" << m_OrigLocation << "\n").Write();
    (log << "   prov: '" << m_Provider<< "'\n").Write();
  }
};

class ChannelList
{
public:
  ChannelList()
  {
    reset();
  }
  
  void reset()
  {
    m_List.clear();
  }

  int size() const
  {
    return m_List.size();
  }

  void add(Channel& channel)
  {
    m_List.push_back(channel);
  }

  vector<Channel>::iterator end()
  {
    return m_List.end();
  }

  vector<Channel>::iterator begin()
  {
    return m_List.begin();
  }

private:
  vector<Channel> m_List;
};

#define LOCK_MODE_LOCKED	0
#define LOCK_MODE_QUALITY	1
#define LOCK_MODE_COUNT		2

class BDAChannelScan : IGuideDataEvent, public LogMessageCaller
{
public:
	BDAChannelScan();
	~BDAChannelScan();
	
	HRESULT CreateGraph();
	HRESULT	BuildGraph();
	HRESULT	ConnectGraph();
	HRESULT	LockChannel(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime);
	HRESULT	LockChannelFast(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime);
	HRESULT	SignalStatsCont(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime);
	HRESULT	SignalStatsCont2(long lFrequency, long lBandwidth, BOOL &locked, BOOL &present, long &strength, long &quality, long &sampletime);
	HRESULT GetSignalStatistics(BOOL &locked, BOOL &present, long &strength, long &quality, long&sampletime);

	HRESULT selectCard();
	void AddNetwork(long freq, long band);
	void AddSummary(long freq, long strength, long quality, unsigned char * transmitter, long streamid);
	HRESULT scanAllUK();
	HRESULT scanAllUK166();
	HRESULT scanAllAus();
	HRESULT scanAllEuro();
	HRESULT scanAllTaiw();
  void ShowChannelResults();

	HRESULT SignalStatisticsAveraged(long frequency, long bandwidth);
	HRESULT SignalStatisticsScrolling(long frequency, long bandwidth);

	BOOL IsVerbose() { return m_bVerbose; }
	void ToggleVerbose();

	long LockDetectionMode() { return m_nLockDetectionMode; }
	void ToggleLockDetectionMode();

  long SageTVOutputChannelList() { return m_bSageTVOutputChannelList; }
  void ToggleSageTVOutput(){ m_bSageTVOutputChannelList = !m_bSageTVOutputChannelList;}

  long SageTVUseExactFreq() { return m_bSageTVUseExactFrequency; }
  void ToggleSageTVExactFreq(){ m_bSageTVUseExactFrequency = !m_bSageTVUseExactFrequency;}

  long SageTVIncludeRadio() { return m_bSageTVIncludeRadio; }
  void ToggleSageTVIncludeRadio(){ m_bSageTVIncludeRadio = !m_bSageTVIncludeRadio;}

  long SageTVSortByLCN() { return m_bSageTVSortByLCN; }
  void ToggleSageTVSorting(){ m_bSageTVSortByLCN = !m_bSageTVSortByLCN;}

	long SageTVMoveDuplicates() { return m_bSageTVMoveDuplicates; }
	void ToggleSageTVMoveduplicates() { m_bSageTVMoveDuplicates = ! m_bSageTVMoveDuplicates; }

	BOOL StartGraph();
	BOOL StopGraph();

	STDMETHODIMP_(ULONG) AddRef(void);
	STDMETHODIMP_(ULONG) Release(void);
	STDMETHODIMP QueryInterface(REFIID, void **);
	
	STDMETHODIMP ProgramChanged(VARIANT varProgramDescriptionID);
	STDMETHODIMP ServiceChanged(VARIANT varServiceDescriptionID);
	STDMETHODIMP ScheduleEntryChanged(VARIANT varScheduleEntryDescriptionID);
	STDMETHODIMP GuideDataAcquired();
	STDMETHODIMP ProgramDeleted(VARIANT varProgramDescriptionID);
	STDMETHODIMP ServiceDeleted(VARIANT varServiceDescriptionID);
	STDMETHODIMP ScheduleDeleted(VARIANT varScheduleEntryDescriptionID);

	HRESULT createConnectionPoint();


private:
	HRESULT SetupGraph();
	void DestroyGraph();

	void VerboseToFileON();
	void VerboseToFileOFF();
	HRESULT scanChannel(long channelNumber, long frequency, long bandwidth);
	CComPtr <IBaseFilter> m_pBDANetworkProvider;

	CComPtr <IBaseFilter> m_pBDAMpeg2Demux;
	CComPtr <IBaseFilter> m_pBDATIF;
	CComPtr <IBaseFilter> m_pBDASecTab;

	CComPtr <IBaseFilter> m_pTSFileSink;
	
	CComPtr <ITuningSpace> m_pTuningSpace;
	CComPtr <IScanningTuner> m_piTuner;
	CComPtr <IGuideData> m_piGuideData;
	CComPtr <IConnectionPoint> m_piConnectionPoint;
	CComPtr <IGraphBuilder> m_piGraphBuilder;
	CComPtr <IMediaControl> m_piMediaControl;

	FilterGraphTools graphTools;
	Mpeg2DataParser m_mpeg2parser;

	LogMessage verbose;
	LogMessageConsoleOutput m_console;
	int m_consoleHandle;
	int m_consoleVerboseHandle;
	int m_logfileHandle;
	int m_logfileVerboseHandle;

	BDACardCollection cardList;
	BDACard *m_pBDACard;

	DWORD m_dwAdviseCookie;
	DWORD m_rotEntry;

	ULONG m_ulRef;

	BOOL m_bServiceChanged;
	BOOL m_bProgramChanged;
	BOOL m_bScheduleEntryChanged;

	HANDLE m_hGuideDataChangedEvent;
	HANDLE m_hServiceChangedMutex;

	BOOL m_bScanning;
	BOOL m_bVerbose;
	long m_nLockDetectionMode;

	long m_freq[256];
	long m_band[256];
	long m_Count;

	long summ_freq[256];
	long summ_strength[256];
	long summ_quality[256];
	long summ_streamid[256];
	unsigned char * summ_transmitter[256];
	long summ_Count;

  bool m_bSageTVOutputChannelList;
  ChannelList m_ChannelList;
  bool m_bSageTVIncludeRadio;
  bool m_bSageTVUseExactFrequency;
  bool m_bSageTVSortByLCN;
	bool m_bSageTVMoveDuplicates;
};

#endif

